unit fSimplifiedModel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, atMath;

type
  TfmSimplifiedModel = class(TForm)
    TreeView1: TTreeView;
    procedure FormDeactivate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FMathEvaluator : TatMathEvaluator;
  public
    { Public declarations }
    property MathEvaluator:TatMathEvaluator read FMathEvaluator write FMathEvaluator;
  end;

var
  fmSimplifiedModel: TfmSimplifiedModel;

implementation

{$R *.DFM}

procedure TfmSimplifiedModel.FormDeactivate(Sender: TObject);
begin
   Close;
end;

procedure TfmSimplifiedModel.FormKeyPress(Sender: TObject; var Key: Char);
begin
   if word(key)=VK_ESCAPE then
   begin
      Close;
      key:=#0;
   end;
end;

type
   THackEquation = class(TEquation)
   end;

procedure TfmSimplifiedModel.FormShow(Sender: TObject);
var c           : integer;
    node, node1 : TTreeNode;
    Instruction : pInstruction;

   function AddNode( AParent:TTreeNode; ACaption:string ):TTreeNode;
   begin
      result:=TreeView1.Items.AddChild( AParent,ACaption );
      { for future use }
   end;

begin
   { build the math evaluator structure }
   with TreeView1, FMathEvaluator do
   begin
      Items.BeginUpdate;
      try
         Items.Clear;
         { show all constants }
         node:=AddNode(nil,'Constants');
         for c:=0 to Constants.Count-1 do
            AddNode(node,Format('%s = (%s,%s)',[
               Constants[c].Id,
               FloatToStr(Constants[c].a),
               FloatToStr(Constants[c].b)]) );
         { show all operators }
         node:=AddNode(nil,'Operators');
         for c:=0 to Operators.Count-1 do
            AddNode(node,Format('%s (level %d)',[
               Operators[c].Id,
               Operators[c].Level]) );
         { show all functions }
         node:=AddNode(nil,'Functions');
         for c:=0 to Functions.Count-1 do
            AddNode(node,Format('%s (%d parameters)',[
               Functions[c].Id,
               Functions[c].Params]) );
         { show all variables }
         node:=AddNode(nil,'Variables');
         for c:=0 to Variables.Count-1 do
            AddNode(node,Format('%s',[
               Variables[c].Id ]) );
         { show all equations }
         node:=AddNode(nil,'Equations');
         for c:=0 to Equations.Count-1 do
            if Equations[c].EquationStr>'' then
            begin
               node1:=AddNode(node,Equations[c].EquationStr);
               { show simplified model }
               Instruction:=THackEquation(Equations[c]).Instructions;
               while (Instruction<>nil) do
                  with Instruction^ do
                  begin
                     case Instruction^.Operation of
                        opRealNum:       AddNode( node1, Format('Real number: %s',[FloatToStr(Value)]) );
                        opImaginaryNum:  AddNode( node1, Format('Imaginary number: %s',[FloatToStr(Value)]) );
                        opOperator:      AddNode( node1, Format('Operator: %s',[Operators[Index].Id]) );
                        opConstant:      AddNode( node1, Format('Constant: %s',[Constants[Index].Id]) );
                        opVariable:      AddNode( node1, Format('Variable: %s',[Variables[Index].Id]) );
                        opLocalVar:      AddNode( node1, Format('Local variable: %s',[THackEquation(Equations[c]).LocalVars[Index].Id]) );
                        opFunction:      AddNode( node1, Format('Function: %s',[Functions[Index].Id]) );
                        opLocalFunction: AddNode( node1, Format('Local function: %s',[Equations[Index].FunctionName]) );
                     end;
                     Instruction:=Next;
                  end;
            end;
      finally
         Items.EndUpdate;
      end;
      FullCollapse;
   end;
end;

end.
